(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: configuration.ml,v 1.21 2014-02-26 23:33:54 fclement Exp $ *)

module Nat = Unatural;;

let c_qual = Upervasives.qualify "Configuration";;

let c_failwith = Upervasives.failwith_qual c_qual;;
let c_invalid_arg = Upervasives.invalid_arg_qual c_qual "";;

open Refinement_types;;
open Options_types;;
open Options;;
open Configuration_types;;

type t = Configuration_types.t;;

(* Uhistory.Make functor interface. *)

let print = Configuration_types_print.print_t;;

let get_external_program_command () =
  match Options.get_option_external_program_command () with
  | [] -> c_invalid_arg "empty external program command path"
  | _cmd :: _args as sl -> String.concat " " sl
;;

let get_dimension () = Nat.of_int (get_option_dimension ());;

let get_dimension_management = get_option_dimension_management;;

let get_cutting_strategy () =
  match get_option_cutting_strategy () with
  | Option_Best_in_a_family ->
      let nct = Nat.of_int (get_option_number_of_cuts ()) in
      let bcfl = get_option_basic_cutting_families () in
      Best_in_a_family (nct, Ulist1.of_list bcfl)
  | Option_Overall_best -> Overall_best
;;

let get_candidate_cuttings_selection_criterion () =
  let r = get_option_candidate_cuttings_selection_criterion_ratio ()
  and n =
    get_option_candidate_cuttings_selection_criterion_maximum_number () in
  if r < 0. || r > 1. then
    c_failwith "get_candidate_cuttings_selection_criterion"
      "ratio is outside [0,1]" else
  if n <= 0 then
    c_failwith "get_candidate_cuttings_selection_criterion"
      "maximum number is nonpositive" else
  (r, Nat.of_int n)
;;

let get_stopping_criterion () =
  {
   max_iteration = Nat.of_int (get_option_max_iteration ());
   min_best_scalar_indicator = get_option_min_best_scalar_indicator ();
   min_scalar_relative_error = get_option_min_scalar_relative_error ();
  }
;;

let { Uaref.get = get_configuration_opt; set = set_configuration_opt; } =
  Uaref.make_aref_unique "algorithm configuration" None
;;

let cold_set () =
  let config =
    {
     external_program_command = get_external_program_command ();
     dimension = get_dimension ();
     dimension_management = get_dimension_management ();
     cutting_strategy = get_cutting_strategy ();
     candidate_cuttings_selection_criterion =
       get_candidate_cuttings_selection_criterion ();
     stopping_criterion = get_stopping_criterion ();
    } in
  set_configuration_opt (Some config)
;;

let warm_set config =
  let config =
    { config with stopping_criterion = get_stopping_criterion (); } in
  set_configuration_opt (Some config)
;;

let get () =
  match get_configuration_opt () with
  | None -> assert false
  | Some config -> config
;;

(* Algorithm configuration. *)

let phase_dimension = Options.phase_dimension;;

let get_external_program_command () = (get ()).external_program_command;;
let get_dimension () = (get ()).dimension;;
let get_dimension_management () = (get ()).dimension_management;;
let get_cutting_strategy () = (get ()).cutting_strategy;;
let get_candidate_cuttings_selection_criterion () =
  (get ()).candidate_cuttings_selection_criterion;;
let get_stopping_criterion () = (get ()).stopping_criterion;;

(* Program configuration. *)

let get_formatter () =
  match get_option_out_formatter () with
  | Option_Std_out -> Format.std_formatter
  | Option_Std_err -> Format.err_formatter
  | Option_Out_file fname ->
      Format.formatter_of_out_channel (Pervasives.open_out fname)
;;

let get_report_allocated_memory = get_option_report_allocated_memory;;

let get_incremental_mode = get_option_incremental_mode;;

let get_graphics_mode = get_option_graphics_mode;;

let check_graphics_window_sizes () =
  let w_min = get_option_graphics_window_min_width ()
  and h_min = get_option_graphics_window_min_height () in
  let w_max = get_option_graphics_window_max_width ()
  and h_max = get_option_graphics_window_max_height () in
  if w_max < w_min then
    c_invalid_arg "Min width is greater than max width" else
  if h_max < h_min then
    c_invalid_arg "Min height is greater then max height"
;;

let get_graphics_window_min_size () =
  check_graphics_window_sizes ();
  let w_min = get_option_graphics_window_min_width ()
  and h_min = get_option_graphics_window_min_height () in
  (Nat.of_int w_min, Nat.of_int h_min)
;;

let get_graphics_window_max_size () =
  check_graphics_window_sizes ();
  let w_max = get_option_graphics_window_max_width ()
  and h_max = get_option_graphics_window_max_height () in
  (Nat.of_int w_max, Nat.of_int h_max)
;;

let num_steps = 256;;

let make_colormapping = function
  | Gray -> Colormapping.make_rgb Graphics.black Graphics.white
  | Cool -> Colormapping.make_rgb Graphics.cyan Graphics.magenta
  | Hot -> Colormapping.make_hsb Graphics.red Graphics.yellow
  | Jet -> Colormapping.make_hsb Graphics.red Graphics.blue
;;

let get_graphics_parameter_color_mapping () =
  let csn = get_option_graphics_parameter_color_scale_name () in
  let min = get_option_graphics_min_parameter ()
  and max = get_option_graphics_max_parameter () in
  if max < min then
    c_failwith "get_graphics_parameter_color_mapping"
      "min parameter is greater than max parameter" else
  make_colormapping csn num_steps min max
;;

let get_graphics_gradient_color_mapping () =
  let csn = get_option_graphics_gradient_color_scale_name () in
  let max = get_option_graphics_gradient_amplitude () in
  if max < 0. then
    c_failwith "get_graphics_gradient_color_mapping"
      "gradient amplitude is negative" else
  let min = -. max in
  make_colormapping csn num_steps min max
;;

(* Monitoring. *)

let {
  Udebug.report = report;
  debug = debug;
  warning = warning;
  error = error;
  report_action = report_action;
  debug_action = debug_action;
  warning_action = warning_action;
  error_action = error_action;
} = Options.printing_facilities
;;

let print_ident_value ident printer ppf value =
  Format.fprintf ppf "%s =@ %a" ident printer value
;;

let print_report printer ppf value =
  report_action (fun () -> Format.fprintf ppf "@[<hv>%a@]" printer value; "")
;;

let print_debug fun_name ident printer ppf value =
  debug_action fun_name
    (fun () ->
      Format.fprintf ppf "@[<v 2>%s:@ %a@]@."
        fun_name (print_ident_value ident printer) value; "")
;;

let print_debug_no_arg fun_name printer ppf =
  debug_action fun_name
    (fun () -> Format.fprintf ppf "@[<v 2>%s:@ %t@]@." fun_name printer; "")
;;
